module Deveo
  class App

    INITIAL_VERSION = "0.0.1"

    def self.package(path)
      Zip::File.open("#{File.basename(path)}.zip", Zip::File::CREATE) do |zipfile|
        Dir[File.join(path, "**", "**")].each do |file|
          zipfile.add(file.sub(File.join(path, ""), ""), file)
        end
      end
    end

    attr_reader :name, :path, :meta

    def initialize(name)
      @name = name
      @path = "#{Dir.pwd}/#{name}"
      @meta = Metadata.new(@path)
    end

    def init
      copy_skeleton
      write_metadata
    end

    private

    def copy_skeleton
      Skeleton.copy_to(@path)
    end

    def write_metadata
      @meta.write(
        id: @name.downcase,
        name: @name,
        version: INITIAL_VERSION,
        plugin_key: SecureRandom.hex(16)
      )
    end

    class Metadata

      def initialize(app_path)
        @app_path = app_path
      end

      def filename
        File.join(@app_path, "app.json")
      end

      def read
        JSON.parse(File.read(filename), symbolize_names: true)
      end

      def write(data = {})
        data = read.merge(data)
        File.open(filename, "w") do |file|
          file.write(JSON.pretty_generate(data))
        end
      end

    end

  end
end
